@@ -48,9 +48,11 @@
 #include "vlandevice.h"
 #include "wireddevice.h"
 #include "wirelessdevice.h"
-#if !NM_CHECK_VERSION(1, 2, 0)
 #include "wimaxdevice.h"
-#endif
+#include "gredevice.h"
+#include "macvlandevice.h"
+#include "tundevice.h"
+#include "vethdevice.h"
 
 #include "nmdebug.h"
 
@@ -80,14 +82,13 @@ NetworkManager::NetworkManagerPrivate::NetworkManagerPrivate()
     , nmState(NetworkManager::Unknown)
     , m_connectivity(NetworkManager::UnknownConnectivity)
     , m_isNetworkingEnabled(false)
-#if !NM_CHECK_VERSION(1, 2, 0)
     , m_isWimaxEnabled(false)
     , m_isWimaxHardwareEnabled(false)
-#endif
     , m_isWirelessEnabled(false)
     , m_isWirelessHardwareEnabled(false)
     , m_isWwanEnabled(false)
     , m_isWwanHardwareEnabled(false)
+    , m_supportedInterfaceTypes(NetworkManager::Device::UnknownType)
 {
     QLoggingCategory::setFilterRules(QStringLiteral("networkmanager-qt.debug = true"));
     QLoggingCategory::setFilterRules(QStringLiteral("networkmanager-qt.warning = true"));
@@ -96,9 +97,23 @@ NetworkManager::NetworkManagerPrivate::NetworkManagerPrivate()
             this, &NetworkManagerPrivate::onDeviceAdded);
     connect(&iface, &OrgFreedesktopNetworkManagerInterface::DeviceRemoved,
             this, &NetworkManagerPrivate::onDeviceRemoved);
+
+
+#ifndef NMQT_STATIC
+#if NM_CHECK_VERSION(1, 4, 0)
+    QDBusConnection::systemBus().connect(NetworkManagerPrivate::DBUS_SERVICE, NetworkManagerPrivate::DBUS_DAEMON_PATH, NetworkManagerPrivate::FDO_DBUS_PROPERTIES,
+                                         QLatin1String("PropertiesChanged"), this, SLOT(dbusPropertiesChanged(QString,QVariantMap,QStringList)));
+#else
     connect(&iface, &OrgFreedesktopNetworkManagerInterface::PropertiesChanged,
             this, &NetworkManagerPrivate::propertiesChanged);
+#endif
+#endif
 
+#ifdef NMQT_STATIC
+    connect(&iface, &OrgFreedesktopNetworkManagerInterface::PropertiesChanged,
+            this, &NetworkManagerPrivate::propertiesChanged);
+#endif
+   
     connect(&watcher, &QDBusServiceWatcher::serviceRegistered,
             this, &NetworkManagerPrivate::daemonRegistered);
     connect(&watcher, &QDBusServiceWatcher::serviceUnregistered,
@@ -139,25 +154,31 @@ void NetworkManager::NetworkManagerPrivate::init()
 
     m_version = iface.version();
     parseVersion(m_version);
+    m_supportedInterfaceTypes =  static_cast<NetworkManager::Device::Types>(
+               NetworkManager::Device::Ethernet |
+               NetworkManager::Device::Wifi |
+               NetworkManager::Device::Modem |
+               (checkVersion(1, 2, 0) ? 0 : NetworkManager::Device::Wimax) |
+               NetworkManager::Device::Bluetooth |
+               NetworkManager::Device::OlpcMesh |
+               NetworkManager::Device::InfiniBand |
+               NetworkManager::Device::Bond |
+               NetworkManager::Device::Vlan |
+               NetworkManager::Device::Adsl |
+               NetworkManager::Device::Bridge |
+               NetworkManager::Device::Generic |
+               NetworkManager::Device::Team
+           );
 
     // Get all Manager's properties async
-    QDBusMessage message = QDBusMessage::createMethodCall(DBUS_SERVICE,
-                           DBUS_DAEMON_PATH,
-                           FDO_DBUS_PROPERTIES,
-                           QLatin1String("GetAll"));
-    message << iface.staticInterfaceName();
-#ifdef NMQT_STATIC
-    QDBusConnection::sessionBus().callWithCallback(message,
-#else
-    QDBusConnection::systemBus().callWithCallback(message,
-#endif
-            this,
-            SLOT(propertiesChanged(QVariantMap)));
+    QVariantMap initialProperties = retrieveInitialProperties(iface.staticInterfaceName(), DBUS_DAEMON_PATH);
+    if (!initialProperties.isEmpty()) {
+        propertiesChanged(initialProperties);
+    }
 
-    qobject_cast<SettingsPrivate *>(settingsNotifier())->init();
+    QTimer::singleShot(0, [this] { qobject_cast<SettingsPrivate *>(settingsNotifier())->init(); });
 
     if (iface.isValid()) {
-#if NM_CHECK_VERSION(0, 9, 10)
         QList <QDBusObjectPath> devices = iface.devices();
         qCDebug(NMQT) << "Device list";
         Q_FOREACH (const QDBusObjectPath & op, devices) {
@@ -165,20 +186,6 @@ void NetworkManager::NetworkManagerPrivate::init()
             Q_EMIT deviceAdded(op.path());
             qCDebug(NMQT) << "  " << op.path();
         }
-#else
-        QDBusReply< QList <QDBusObjectPath> > deviceList = iface.GetDevices();
-        if (deviceList.isValid()) {
-            qCDebug(NMQT) << "Device list";
-            QList <QDBusObjectPath> devices = deviceList.value();
-            Q_FOREACH (const QDBusObjectPath & op, devices) {
-                networkInterfaceMap.insert(op.path(), Device::Ptr());
-                Q_EMIT deviceAdded(op.path());
-                qCDebug(NMQT) << "  " << op.path();
-            }
-        } else {
-            qCDebug(NMQT) << "Error getting device list: " << deviceList.error().name() << ": " << deviceList.error().message();
-        }
-#endif
     }
 }
 
@@ -229,6 +236,40 @@ int NetworkManager::NetworkManagerPrivate::compareVersion(const int x, const int
     return 0;
 }
 
+bool NetworkManager::NetworkManagerPrivate::checkVersion(const int x, const int y, const int z) const
+{
+    return 0 <= compareVersion(x, y, z);
+}
+
+NetworkManager::Device::Types NetworkManager::NetworkManagerPrivate::supportedInterfaceTypes() const
+{
+    return m_supportedInterfaceTypes;
+}
+
+QVariantMap NetworkManager::NetworkManagerPrivate::retrieveInitialProperties(const QString &interfaceName, const QString &path)
+{
+    QDBusMessage message = QDBusMessage::createMethodCall(DBUS_SERVICE,
+                                                          path,
+                                                          FDO_DBUS_PROPERTIES,
+                                                          QLatin1String("GetAll"));
+    message << interfaceName;
+#ifdef NMQT_STATIC
+    QDBusMessage resultMessage = QDBusConnection::sessionBus().call(message);
+#else
+    QDBusMessage resultMessage = QDBusConnection::systemBus().call(message);
+#endif
+    if (resultMessage.type() == QDBusMessage::ReplyMessage) {
+        QVariantMap result;
+        QDBusArgument dbusArgument = resultMessage.arguments().at(0).value<QDBusArgument>();
+        while (!dbusArgument.atEnd()) {
+            dbusArgument >> result;
+        }
+        return result;
+    }
+
+    return QVariantMap();
+}
+
 NetworkManager::Device::Ptr NetworkManager::NetworkManagerPrivate::findRegisteredNetworkInterface(const QString &uni)
 {
     NetworkManager::Device::Ptr networkInterface;
@@ -283,11 +324,9 @@ NetworkManager::Device::Ptr NetworkManager::NetworkManagerPrivate::createNetwork
     case Device::Bluetooth:
         createdInterface = Device::Ptr(new NetworkManager::BluetoothDevice(uni), &QObject::deleteLater);
         break;
-#if !NM_CHECK_VERSION(1, 2, 0)
     case Device::Wimax:
         createdInterface = Device::Ptr(new NetworkManager::WimaxDevice(uni), &QObject::deleteLater);
         break;
-#endif
     case Device::OlpcMesh:
         createdInterface = Device::Ptr(new NetworkManager::OlpcMeshDevice(uni), &QObject::deleteLater);
         break;
@@ -306,11 +345,22 @@ NetworkManager::Device::Ptr NetworkManager::NetworkManagerPrivate::createNetwork
     case Device::Bridge:
         createdInterface = Device::Ptr(new NetworkManager::BridgeDevice(uni), &QObject::deleteLater);
         break;
-#if NM_CHECK_VERSION(0, 9, 10)
+    //No need to check checkVersion, because we can't get Generic, Gre, MacVlan, Tun & Veth values in incompatible runtime
     case Device::Generic:
         createdInterface = Device::Ptr(new NetworkManager::GenericDevice(uni), &QObject::deleteLater);
         break;
-#endif
+    case Device::Gre:
+        createdInterface = Device::Ptr(new NetworkManager::GreDevice(uni), &QObject::deleteLater);
+        break;
+    case Device::MacVlan:
+        createdInterface = Device::Ptr(new NetworkManager::MacVlanDevice(uni), &QObject::deleteLater);
+        break;
+    case Device::Tun:
+        createdInterface = Device::Ptr(new NetworkManager::TunDevice(uni), &QObject::deleteLater);
+        break;
+    case Device::Veth:
+        createdInterface = Device::Ptr(new NetworkManager::VethDevice(uni), &QObject::deleteLater);
+        break;
     default:
         createdInterface = device;
         if (uni != QLatin1String("any")) { // VPN connections use "any" as uni for the network interface.
@@ -382,17 +432,15 @@ bool NetworkManager::NetworkManagerPrivate::isWwanHardwareEnabled() const
     return m_isWwanHardwareEnabled;
 }
 
-#if !NM_CHECK_VERSION(1, 2, 0)
 bool NetworkManager::NetworkManagerPrivate::isWimaxEnabled() const
 {
-    return m_isWimaxEnabled;
+    return checkVersion(1, 2, 0) ? false : m_isWimaxEnabled;
 }
 
 bool NetworkManager::NetworkManagerPrivate::isWimaxHardwareEnabled() const
 {
-    return m_isWimaxHardwareEnabled;
+    return checkVersion(1, 2, 0) ? false : m_isWimaxHardwareEnabled;
 }
-#endif
 
 QDBusPendingReply<QDBusObjectPath> NetworkManager::NetworkManagerPrivate::activateConnection(const QString &connectionUni, const QString &interfaceUni, const QString &connectionParameter)
 {
@@ -442,12 +490,11 @@ void NetworkManager::NetworkManagerPrivate::setWwanEnabled(bool enabled)
     iface.setWwanEnabled(enabled);
 }
 
-#if !NM_CHECK_VERSION(1, 2, 0)
 void NetworkManager::NetworkManagerPrivate::setWimaxEnabled(bool enabled)
 {
-    iface.setWimaxEnabled(enabled);
+    if (!checkVersion(1, 2, 0))
+        iface.setWimaxEnabled(enabled);
 }
-#endif
 
 void NetworkManager::NetworkManagerPrivate::sleep(bool sleep)
 {
@@ -471,26 +518,17 @@ void NetworkManager::NetworkManagerPrivate::setLogging(NetworkManager::LogLevel
     case NetworkManager::Debug:
         logLevel = QLatin1String("DEBUG");
         break;
-#if NM_CHECK_VERSION(0, 9, 10)
     case NetworkManager::Trace:
         logLevel = QLatin1String("TRACE");
         break;
-#endif
     }
     if (!domains.testFlag(NoChange)) {
         if (domains.testFlag(NetworkManager::None)) {
             logDomains << QLatin1String("NONE");
         }
-#if NM_CHECK_VERSION(0, 9, 10)
         if (domains.testFlag(NetworkManager::Hardware)) {
             logDomains << QLatin1String("PLATFORM");
         }
-#else
-        if (domains.testFlag(NetworkManager::Hardware)) {
-            logDomains << QLatin1String("HW");
-        }
-#endif
-
         if (domains.testFlag(NetworkManager::RFKill)) {
             logDomains << QLatin1String("RFKILL");
         }
@@ -575,7 +613,6 @@ void NetworkManager::NetworkManagerPrivate::setLogging(NetworkManager::LogLevel
         if (domains.testFlag(NetworkManager::Vlan)) {
             logDomains << QLatin1String("VLAN");
         }
-#if NM_CHECK_VERSION(0, 9, 10)
         if (domains.testFlag(NetworkManager::Agents)) {
             logDomains << QLatin1String("AGENTS");
         }
@@ -597,7 +634,6 @@ void NetworkManager::NetworkManagerPrivate::setLogging(NetworkManager::LogLevel
         if (domains.testFlag(NetworkManager::Dispatch)) {
             logDomains << QLatin1String("DISPATCH");
         }
-#endif
     }
     iface.SetLogging(logLevel, logDomains.join(QLatin1String(",")));
 }
@@ -627,26 +663,20 @@ NetworkManager::ActiveConnection::Ptr NetworkManager::NetworkManagerPrivate::act
     return findRegisteredActiveConnection(m_activatingConnection);
 }
 
-#if NM_CHECK_VERSION(1, 0, 0)
 NetworkManager::ConnectionSettings::ConnectionType NetworkManager::NetworkManagerPrivate::primaryConnectionType()
 {
-    return m_primaryConnectionType;
+    return checkVersion(1, 0, 0) ? m_primaryConnectionType : NetworkManager::ConnectionSettings::Unknown;
 }
-#endif
 
-#if NM_CHECK_VERSION(0, 9, 10)
 bool NetworkManager::NetworkManagerPrivate::isStartingUp() const
 {
     return iface.startup();
 }
-#endif
 
-#if NM_CHECK_VERSION(1, 0, 6)
 NetworkManager::Device::MeteredStatus NetworkManager::NetworkManagerPrivate::metered() const
 {
-    return m_metered;
+    return checkVersion(1, 0, 6) ? m_metered : NetworkManager::Device::UnknownStatus;
 }
-#endif
 
 void NetworkManager::NetworkManagerPrivate::onDeviceAdded(const QDBusObjectPath &objpath)
 {
@@ -682,6 +712,14 @@ void NetworkManager::NetworkManagerPrivate::stateChanged(uint state)
     }
 }
 
+void NetworkManager::NetworkManagerPrivate::dbusPropertiesChanged(const QString &interfaceName, const QVariantMap &properties, const QStringList &invalidatedProperties)
+{
+    Q_UNUSED(invalidatedProperties);
+    if (interfaceName == QLatin1String("org.freedesktop.NetworkManager")) {
+        propertiesChanged(properties);
+    }
+}
+
 void NetworkManager::NetworkManagerPrivate::propertiesChanged(const QVariantMap &changedProperties)
 {
     // qCDebug(NMQT) << Q_FUNC_INFO << changedProperties;
@@ -735,7 +773,6 @@ void NetworkManager::NetworkManagerPrivate::propertiesChanged(const QVariantMap
             m_isWwanEnabled = it->toBool();
             qCDebug(NMQT) << property << m_isWwanEnabled;
             Q_EMIT wwanEnabledChanged(m_isWwanEnabled);
-#if !NM_CHECK_VERSION(1, 2, 0)
         } else if (property == QLatin1String("WimaxHardwareEnabled")) {
             m_isWimaxHardwareEnabled = it->toBool();
             qCDebug(NMQT) << property << m_isWimaxHardwareEnabled;
@@ -744,7 +781,6 @@ void NetworkManager::NetworkManagerPrivate::propertiesChanged(const QVariantMap
             m_isWimaxEnabled = it->toBool();
             qCDebug(NMQT) << property << m_isWimaxEnabled;
             Q_EMIT wimaxEnabledChanged(m_isWimaxEnabled);
-#endif
         } else if (property == QLatin1String("Version")) {
             m_version = it->toString();
             parseVersion(m_version);
@@ -758,20 +794,14 @@ void NetworkManager::NetworkManagerPrivate::propertiesChanged(const QVariantMap
         } else if (property == QLatin1String("ActivatingConnection")) {
             m_activatingConnection = it->value<QDBusObjectPath>().path();
             Q_EMIT activatingConnectionChanged(m_activatingConnection);
-#if NM_CHECK_VERSION(1, 0, 0)
         } else if (property == QLatin1String("PrimaryConnectionType")) {
             m_primaryConnectionType = NetworkManager::ConnectionSettings::typeFromString(it->toString());
             Q_EMIT primaryConnectionTypeChanged(m_primaryConnectionType);
-#endif
-#if NM_CHECK_VERSION(0, 9, 10)
         } else if (property == QLatin1String("Startup")) {
             Q_EMIT isStartingUpChanged();
-#endif
-#if NM_CHECK_VERSION(1, 0, 6)
         } else if (property == QLatin1String("Metered")) {
             m_metered = (NetworkManager::Device::MeteredStatus)it->toUInt();
             Q_EMIT meteredChanged(m_metered);
-#endif
         } else {
             qCWarning(NMQT) << Q_FUNC_INFO << "Unhandled property" << property;
         }
@@ -902,6 +932,11 @@ int NetworkManager::compareVersion(const int x, const int y, const int z)
     return globalNetworkManager->compareVersion(x, y, z);
 }
 
+bool NetworkManager::checkVersion(const int x, const int y, const int z)
+{
+    return globalNetworkManager->checkVersion(x, y, z);
+}
+
 NetworkManager::Status NetworkManager::status()
 {
     return globalNetworkManager->status();
@@ -997,7 +1032,6 @@ void NetworkManager::setWwanEnabled(bool enabled)
     globalNetworkManager->setWwanEnabled(enabled);
 }
 
-#if !NM_CHECK_VERSION(1, 2, 0)
 bool NetworkManager::isWimaxEnabled()
 {
     return globalNetworkManager->isWimaxEnabled();
@@ -1012,7 +1046,6 @@ void NetworkManager::setWimaxEnabled(bool enabled)
 {
     globalNetworkManager->setWimaxEnabled(enabled);
 }
-#endif
 
 void NetworkManager::sleep(bool sleep)
 {
@@ -1031,26 +1064,7 @@ NMStringMap NetworkManager::permissions()
 
 NetworkManager::Device::Types NetworkManager::supportedInterfaceTypes()
 {
-    return (NetworkManager::Device::Types)(
-               NetworkManager::Device::Ethernet |
-               NetworkManager::Device::Wifi |
-               NetworkManager::Device::Modem |
-#if !NM_CHECK_VERSION(1, 2, 0)
-               NetworkManager::Device::Wimax |
-#endif
-               NetworkManager::Device::Bluetooth |
-               NetworkManager::Device::OlpcMesh |
-               NetworkManager::Device::InfiniBand |
-               NetworkManager::Device::Bond |
-               NetworkManager::Device::Vlan |
-               NetworkManager::Device::Adsl |
-               NetworkManager::Device::Bridge
-#if NM_CHECK_VERSION(0, 9, 10)
-               |
-               NetworkManager::Device::Generic |
-               NetworkManager::Device::Team
-#endif
-           );
+    return globalNetworkManager->supportedInterfaceTypes();
 }
 
 NetworkManager::Connectivity NetworkManager::connectivity()
@@ -1073,26 +1087,20 @@ NetworkManager::ActiveConnection::Ptr NetworkManager::activatingConnection()
     return globalNetworkManager->activatingConnection();
 }
 
-#if NM_CHECK_VERSION(1, 0, 0)
 NetworkManager::ConnectionSettings::ConnectionType NetworkManager::primaryConnectionType()
 {
     return globalNetworkManager->primaryConnectionType();
 }
-#endif
 
-#if NM_CHECK_VERSION(0, 9, 10)
 bool NetworkManager::isStartingUp()
 {
     return globalNetworkManager->isStartingUp();
 }
-#endif
 
-#if NM_CHECK_VERSION(1, 0, 6)
 NetworkManager::Device::MeteredStatus NetworkManager::metered()
 {
     return globalNetworkManager->metered();
 }
-#endif
 
 NetworkManager::Notifier *NetworkManager::notifier()
 {
